==> pynoder-1.0.1.tar.gz: pynoder-1.0.1/pynoder/mouse_actions.py <==
# Copyright 2017 John Roper
#
# ##### BEGIN GPL LICENSE BLOCK ######
# This file is part of PyNoder.
#
# PyNoder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyNoder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyNoder.  If not, see <http://www.gnu.org/licenses/>.
# ##### END GPL LICENSE BLOCK #####
from PyQt5 import QtGui, QtCore
from port import PortCircle, PortLabel
from connection import Connection


class MouseGrabber(PortCircle):
    """docstring for MouseGrabber"""

    def __init__(self, graph, pos, otherPortCircle, connectionPointType):
        super(MouseGrabber, self).__init__(None, graph, 0, otherPortCircle.getPort().getColor(), connectionPointType)

        self._ellipseItem.setPos(0, 0)
        self._ellipseItem.setStartAngle(0)
        self._ellipseItem.setSpanAngle(360 * 16)

        self.__otherPortItem = otherPortCircle

        self._graph.scene().addItem(self)

        self.setZValue(-1)
        self.setTransform(QtGui.QTransform.fromTranslate(pos.x(), pos.y()), False)
        self.grabMouse()

        if self.connectionPointType() == 'Out':
            self.__connection = Connection(self._graph, self, otherPortCircle)
        elif self.connectionPointType() == 'In':
            self.__connection = Connection(self._graph, otherPortCircle, self)
        # Do not emit a notification for this temporary connection.
        self._graph.addConnection(self.__connection, emitSignal=False)
        self.__mouseOverPortCircle = None
        self._graph.emitBeginConnectionManipulationSignal()

    def getColor(self):
        return self.__otherPortItem.getPort().getColor()

    def mouseMoveEvent(self, event):
        scenePos = self.mapToScene(event.pos())

        for connection in self.getConnections():
            connection.prepareGeometryChange()

        self.setTransform(QtGui.QTransform.fromTranslate(scenePos.x(), scenePos.y()), False)

        collidingItems = self.collidingItems(QtCore.Qt.IntersectsItemBoundingRect)
        collidingPortItems = list(filter(lambda item: isinstance(item, (PortCircle, PortLabel)), collidingItems))

        def canConnect(item):
            if isinstance(item, PortCircle):
                mouseOverPortCircle = item
            else:
                if self.connectionPointType() == 'In':
                    mouseOverPortCircle = item.getPort().inCircle()
                else:
                    mouseOverPortCircle = item.getPort().outCircle()

                if mouseOverPortCircle is None:
                    return False

            return mouseOverPortCircle.canConnectTo(self.__otherPortItem)

        collidingPortItems = list(filter(lambda port: canConnect(port), collidingPortItems))
        if len(collidingPortItems) > 0:

            if isinstance(collidingPortItems[0], PortCircle):
                self.setMouseOverPortCircle(collidingPortItems[0])
            else:
                if self.connectionPointType() == 'In':
                    self.setMouseOverPortCircle(collidingPortItems[0].getPort().inCircle())
                else:
                    self.setMouseOverPortCircle(collidingPortItems[0].getPort().outCircle())

        elif self.__mouseOverPortCircle is not None:
            self.setMouseOverPortCircle(None)

    def mouseReleaseEvent(self, event):

        # Destroy the temporary connection.
        self._graph.removeConnection(self.__connection, emitSignal=False)
        self.__connection = None

        if self.__mouseOverPortCircle is not None:
            try:
                if self.connectionPointType() == 'In':
                    sourcePortCircle = self.__otherPortItem
                    targetPortCircle = self.__mouseOverPortCircle
                elif self.connectionPointType() == 'Out':
                    sourcePortCircle = self.__mouseOverPortCircle
                    targetPortCircle = self.__otherPortItem

                connection = Connection(self._graph, sourcePortCircle, targetPortCircle)
                self._graph.addConnection(connection)
                self._graph.emitEndConnectionManipulationSignal()

            except Exception as e:
                print("Exception in MouseGrabber.mouseReleaseEvent: " + str(e))

            self.setMouseOverPortCircle(None)

        self.destroy()

    def setMouseOverPortCircle(self, portCircle):
        if self.__mouseOverPortCircle != portCircle:
            if self.__mouseOverPortCircle is not None:
                self.__mouseOverPortCircle.unhighlight()
                self.__mouseOverPortCircle.getPort().labelItem().unhighlight()

            self.__mouseOverPortCircle = portCircle

            if self.__mouseOverPortCircle is not None:
                self.__mouseOverPortCircle.highlight()
                self.__mouseOverPortCircle.getPort().labelItem().highlight()

    # def paint(self, painter, option, widget):
    #     super(MouseGrabber, self).paint(painter, option, widget)
    #     painter.setPen(QtGui.QPen(self.getColor()))
    #     painter.drawRect(self.windowFrameRect())

    def destroy(self):
        self.ungrabMouse()
        scene = self.scene()
        if self.__connection is not None:
            self._graph.removeConnection(self.__connection, emitSignal=False)
        # Destroy the grabber.
        scene.removeItem(self)

==> pynoder-1.0.3.tar.gz: pynoder-1.0.3/pynoder/mouse_actions.py <==
# Copyright 2017 John Roper
#
# ##### BEGIN GPL LICENSE BLOCK ######
# This file is part of PyNoder.
#
# PyNoder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyNoder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyNoder.  If not, see <http://www.gnu.org/licenses/>.
# ##### END GPL LICENSE BLOCK #####
from PyQt5 import QtGui, QtCore
from .port import PortCircle, PortLabel
from .connection import Connection


class MouseGrabber(PortCircle):
    """docstring for MouseGrabber"""

    def __init__(self, graph, pos, otherPortCircle, connectionPointType):
        super(MouseGrabber, self).__init__(None, graph, 0, otherPortCircle.getPort().getColor(), connectionPointType)

        self._ellipseItem.setPos(0, 0)
        self._ellipseItem.setStartAngle(0)
        self._ellipseItem.setSpanAngle(360 * 16)

        self.__otherPortItem = otherPortCircle

        self._graph.scene().addItem(self)

        self.setZValue(-1)
        self.setTransform(QtGui.QTransform.fromTranslate(pos.x(), pos.y()), False)
        self.grabMouse()

        if self.connectionPointType() == 'Out':
            self.__connection = Connection(self._graph, self, otherPortCircle)
        elif self.connectionPointType() == 'In':
            self.__connection = Connection(self._graph, otherPortCircle, self)
        # Do not emit a notification for this temporary connection.
        self._graph.addConnection(self.__connection, emitSignal=False)
        self.__mouseOverPortCircle = None
        self._graph.emitBeginConnectionManipulationSignal()

    def getColor(self):
        return self.__otherPortItem.getPort().getColor()

    def mouseMoveEvent(self, event):
        scenePos = self.mapToScene(event.pos())

        for connection in self.getConnections():
            connection.prepareGeometryChange()

        self.setTransform(QtGui.QTransform.fromTranslate(scenePos.x(), scenePos.y()), False)

        collidingItems = self.collidingItems(QtCore.Qt.IntersectsItemBoundingRect)
        collidingPortItems = list(filter(lambda item: isinstance(item, (PortCircle, PortLabel)), collidingItems))

        def canConnect(item):
            if isinstance(item, PortCircle):
                mouseOverPortCircle = item
            else:
                if self.connectionPointType() == 'In':
                    mouseOverPortCircle = item.getPort().inCircle()
                else:
                    mouseOverPortCircle = item.getPort().outCircle()

                if mouseOverPortCircle is None:
                    return False

            return mouseOverPortCircle.canConnectTo(self.__otherPortItem)

        collidingPortItems = list(filter(lambda port: canConnect(port), collidingPortItems))
        if len(collidingPortItems) > 0:

            if isinstance(collidingPortItems[0], PortCircle):
                self.setMouseOverPortCircle(collidingPortItems[0])
            else:
                if self.connectionPointType() == 'In':
                    self.setMouseOverPortCircle(collidingPortItems[0].getPort().inCircle())
                else:
                    self.setMouseOverPortCircle(collidingPortItems[0].getPort().outCircle())

        elif self.__mouseOverPortCircle is not None:
            self.setMouseOverPortCircle(None)

    def mouseReleaseEvent(self, event):

        # Destroy the temporary connection.
        self._graph.removeConnection(self.__connection, emitSignal=False)
        self.__connection = None

        if self.__mouseOverPortCircle is not None:
            try:
                if self.connectionPointType() == 'In':
                    sourcePortCircle = self.__otherPortItem
                    targetPortCircle = self.__mouseOverPortCircle
                elif self.connectionPointType() == 'Out':
                    sourcePortCircle = self.__mouseOverPortCircle
                    targetPortCircle = self.__otherPortItem

                connection = Connection(self._graph, sourcePortCircle, targetPortCircle)
                self._graph.addConnection(connection)
                self._graph.emitEndConnectionManipulationSignal()

            except Exception as e:
                print("Exception in MouseGrabber.mouseReleaseEvent: " + str(e))

            self.setMouseOverPortCircle(None)

        self.destroy()

    def setMouseOverPortCircle(self, portCircle):
        if self.__mouseOverPortCircle != portCircle:
            if self.__mouseOverPortCircle is not None:
                self.__mouseOverPortCircle.unhighlight()
                self.__mouseOverPortCircle.getPort().labelItem().unhighlight()

            self.__mouseOverPortCircle = portCircle

            if self.__mouseOverPortCircle is not None:
                self.__mouseOverPortCircle.highlight()
                self.__mouseOverPortCircle.getPort().labelItem().highlight()

    # def paint(self, painter, option, widget):
    #     super(MouseGrabber, self).paint(painter, option, widget)
    #     painter.setPen(QtGui.QPen(self.getColor()))
    #     painter.drawRect(self.windowFrameRect())

    def destroy(self):
        self.ungrabMouse()
        scene = self.scene()
        if self.__connection is not None:
            self._graph.removeConnection(self.__connection, emitSignal=False)
        # Destroy the grabber.
        scene.removeItem(self)
